/**
 *
 * @author  Jesper Kristensen
 * @Version $Revision: 1.23 $
 */
package dk.auc.cs.whiner.gui.applicant;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

import java.rmi.RemoteException;
import dk.auc.cs.whiner.rmi.RMIException;
import dk.auc.cs.whiner.dataaccess.DAOException;

import dk.auc.cs.whiner.model.*;
import dk.auc.cs.whiner.interfaces.*;
import dk.auc.cs.whiner.gui.Login;
import dk.auc.cs.whiner.gui.share.*;

public class ViewAppApplicantGUI extends JApplet {  
    private MsgErrorWarning msgErrorWarning;
    private Container cp;
    private JPanel mainJPanel;
    private JPanel topPanel;
    private JPanel westPanel;
    private JPanel mainTopPanel;
    private JPanel mainPanel;
    private JPanel buttomPanel;
    private JPanel eastPanel;
    private JLabel titleLabel;
    private JButton logoutButton;
    private JScrollPane applicationScrollPanel;
    private JTextArea applicationTextArea;
    private JButton backButton;
    private JButton saveButton;
    private JButton sendButton;
    private JButton cancelButton;
    private String jobName;
    private int applicationNo;
    private String screenType;
    private ApplicantI applicant;
    private JobI job;

    public ViewAppApplicantGUI(String screenType, int applicationNo, JobI job, ApplicantI applicant) {
	this.applicationNo = applicationNo;
	this.applicant = applicant;
	this.screenType = screenType;
	this.job = job;
        initComponents();
    }

    private void initComponents() {
	applicationScrollPanel = new JScrollPane();
	applicationTextArea = new JTextArea();
	backButton = new JButton();
        buttomPanel = new JPanel();
	cancelButton = new JButton();
	cp = getContentPane();
        eastPanel = new JPanel();
	logoutButton = new JButton();
	mainJPanel = new JPanel();
        mainPanel = new JPanel();
        mainTopPanel = new JPanel();
	saveButton = new JButton();
	sendButton = new JButton();
	titleLabel = new JLabel();
        topPanel = new JPanel();
        westPanel = new JPanel();
	jobName = new String();

	mainJPanel.setLayout(new BorderLayout());

        mainPanel.setLayout(new AbsoluteLayout());

        mainTopPanel.setLayout(new AbsoluteLayout());

        mainTopPanel.setBorder(new SoftBevelBorder(BevelBorder.RAISED, Color.lightGray, Color.lightGray, Color.darkGray, Color.darkGray));

	titleLabel.setText("Application:" );
	mainTopPanel.add(titleLabel, new AbsoluteConstraints(20, 20, -1, -1));

	applicationScrollPanel.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        applicationScrollPanel.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        applicationTextArea.setEditable(false);
        applicationTextArea.setLineWrap(true);
	applicationTextArea.setBackground(new Color(233,233,242));
        applicationTextArea.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent evt) {
		    mouseApplicationTextArea();
		}
	    });
        applicationScrollPanel.setViewportView(applicationTextArea);
        mainTopPanel.add(applicationScrollPanel, new AbsoluteConstraints(20, 40, 520, 385));

	mainTopPanel.setBackground(new Color(233,233,242));
	mainPanel.add(mainTopPanel, new AbsoluteConstraints(2, 12, 566, 450));

	mainPanel.setBackground(new Color(233,233,242));
        mainJPanel.add(mainPanel, BorderLayout.CENTER);

        topPanel.setLayout(new BorderLayout());
        logoutButton.setText("Logout");
        logoutButton.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent evt) {
		    mouseLogoutButton();
		}
	    });
        topPanel.add(logoutButton, BorderLayout.EAST);

	topPanel.setBackground(new Color(233,233,242));
        mainJPanel.add(topPanel, BorderLayout.NORTH);

        buttomPanel.setLayout(new AbsoluteLayout());
	
	backButton.setText("Back to main");
        backButton.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent evt) {
		    mouseBackButton();
		}
	    });
        buttomPanel.add(backButton,  new AbsoluteConstraints(0, 0, -1, -1));

	sendButton.setText("Send");
	sendButton.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent evt) {
		    mouseSendButton();
		}
	    });
	buttomPanel.add(sendButton,  new AbsoluteConstraints(120, 0, -1, -1));

 	saveButton.setText("Save");
	saveButton.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent evt) {
		    mouseSaveButton();
		}
	    });
	buttomPanel.add(saveButton, new AbsoluteConstraints(190, 0, -1, -1));

	cancelButton.setText("Cancel");
	cancelButton.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent evt) {
		    mouseCancelButton();
		}
	    });
	buttomPanel.add(cancelButton,  new AbsoluteConstraints(257, 0, -1, -1));

	buttomPanel.setBackground(new Color(233,233,242));
        mainJPanel.add(buttomPanel, BorderLayout.SOUTH);

	eastPanel.setBackground(new Color(233,233,242));
        mainJPanel.add(eastPanel, BorderLayout.EAST);

	westPanel.setBackground(new Color(233,233,242));
        mainJPanel.add(westPanel, BorderLayout.WEST);

	mainJPanel.setBackground(new Color(233,233,242));
	cp.add(mainJPanel, BorderLayout.CENTER);

	// Get information for the screen
	getInformation();
    }

    private void getInformation() {
	if (applicationNo == -1) { // new application
	    try {
		job.createApplication(applicant.getID());
		applicationNo = job.getJobApplicationListLength() - 1;
		applicant.initializeApplicantApplicationList();
		jobName = job.getTitle();
	    } catch (RemoteException e) {
		msgErrorWarning = new MsgErrorWarning("There was an error with the RMI network connection.");
		msgErrorWarning.msgError();
		return;
	    } catch (DAOException e) {
		msgErrorWarning = new MsgErrorWarning("There was an error with the database.");
		msgErrorWarning.msgError();
		return;
	    } catch (AlreadyAppliedException e) {
		msgErrorWarning = new MsgErrorWarning("You have already applied for this job.");
		msgErrorWarning.msgError();
		mouseBackButton();
		return;
	    } catch (Exception e) {
		msgErrorWarning = new MsgErrorWarning("Unknown exception: ." + e.getMessage());
		msgErrorWarning.msgError();
		return;   
	    }
	    titleLabel.setText("Title: " + jobName);
	    applicationTextArea.setEditable(true);
	    sendButton.setEnabled(false);
	    cancelButton.setEnabled(false);
	    applicationTextArea.setBackground(new Color(255,255,255));
	} else { // View or edit application
	    try {
		jobName = job.getTitle();
		applicationTextArea.setText(((ApplicationI)applicant.getApplication(applicationNo)).getBodyText());
	    } catch (RemoteException e) {
		msgErrorWarning = new MsgErrorWarning("There was an error with the RMI network connection.");
		msgErrorWarning.msgError();
		return;
	    } catch (Exception e) {
		msgErrorWarning = new MsgErrorWarning("Unknown exception: ." + e.getMessage());
		msgErrorWarning.msgError();
		return;   
	    }
	    // Test to see from where this screen was called
	    if (screenType.equals("view")) {
		titleLabel.setText("Title: " + jobName);
		saveButton.setEnabled(false);
		try {
		    if (!((((ApplicationI)applicant.getApplication(applicationNo)).getStatus()).equals("submitted"))) {
			sendButton.setEnabled(true);
		    } else {
			sendButton.setEnabled(false);
		    }
		} catch (RemoteException e) {
		    msgErrorWarning = new MsgErrorWarning("There was an error with the RMI network connection.");
		    msgErrorWarning.msgError();
		    return;
		} catch (Exception e) {
		    msgErrorWarning = new MsgErrorWarning("Unknown exception: ." + e.getMessage());
		    msgErrorWarning.msgError();
		    return;
		}
		cancelButton.setEnabled(false);
	    } else if (screenType.equals("edit")){
		titleLabel.setText("Title: " + jobName);
		cancelButton.setEnabled(false);
		saveButton.setEnabled(false);
		sendButton.setEnabled(false);
		applicationTextArea.setEditable(true);
		applicationTextArea.setBackground(new Color(255,255,255));
	    }
	}
    }

    private void mouseLogoutButton() {
	msgErrorWarning = new MsgErrorWarning("Do you want to logout!");
	int selected = msgErrorWarning.msgLogout();
	// Check the answer from the user
	if (selected == 0) {
	    cp.removeAll();
	    cp.add(BorderLayout.CENTER, new Login());
	    setContentPane(cp);
    	} else {
	    // The user will not logout
	}	
    }

    private void mouseApplicationTextArea() {
	// Check if the area isenabled in relate to new
	if (applicationTextArea.isEditable() == false) {return;}
	// The application textarea as been activated
	cancelButton.setEnabled(true);
	saveButton.setEnabled(true);
	sendButton.setEnabled(false);
    }

    private void mouseSendButton() {
	// Check button status
	if (sendButton.isEnabled() == false) {return;}	
	try {
	    ((ApplicationI)applicant.getApplication(applicationNo)).submitApplication();
	} catch (RemoteException e) {
	    msgErrorWarning = new MsgErrorWarning("There was an error with the RMI network connection.");
	    msgErrorWarning.msgError();
	    return;
	} catch (DAOException e) {
	    msgErrorWarning = new MsgErrorWarning("There was an error with the database.");
	    msgErrorWarning.msgError();
	    return;
	} catch (Exception e) {
	    msgErrorWarning = new MsgErrorWarning("Unknown exception: ." + e.getMessage());
	    msgErrorWarning.msgError();
	    return;   
	}
	// Change screen
	cp.removeAll();
	cp.add(BorderLayout.CENTER, new MainApplicantGUI(applicant));
	setContentPane(cp);
    }

    private void mouseBackButton() {
	cp.removeAll();
	cp.add(BorderLayout.CENTER, new MainApplicantGUI(applicant));
	setContentPane(cp);
    }

    private void mouseSaveButton() {
	// Check if the button isenabled in relate to new
	if (saveButton.isEnabled() == false) {return;}
	// Check if there has been written any thing in the textarea
	if ((applicationTextArea.getText()).equals("")) {
	    msgErrorWarning = new MsgErrorWarning("You can't make a new application, when no text has been written.");
	    msgErrorWarning.msgError();
	    return;
	}
	// Makes a msg box to verfiy the deletion
	msgErrorWarning = new MsgErrorWarning("Do you want to save the changes!");
	int selected = msgErrorWarning.msgWarning();
	// Check the answer from the user, if it's 0 save the application eles do nothing
	if (selected == 0) {
	    // Make the button's enabled t/f
	    sendButton.setEnabled(true);
	    saveButton.setEnabled(false);
	    cancelButton.setEnabled(false);
	    // Save application
	    try {
		((ApplicationI)applicant.getApplication(applicationNo)).setBodyText(applicationTextArea.getText());
		((ApplicationI)applicant.getApplication(applicationNo)).save();
	    } catch (RemoteException e) {
		msgErrorWarning = new MsgErrorWarning("There was an error with the RMI network connection.");
		msgErrorWarning.msgError();
		return;
	    } catch (DAOException e) {
		msgErrorWarning = new MsgErrorWarning("There was an error with the database.");
		msgErrorWarning.msgError();
		return;
	    } catch (Exception e) {
		msgErrorWarning = new MsgErrorWarning("Unknown exception: ." + e.getMessage());
		msgErrorWarning.msgError();
		return;   
	    }
	} else {
	    return;
	}	
    }

    private void mouseCancelButton() {
	// Check if the button isenabled in relate to new
	if (cancelButton.isEnabled() == false) {return;}
	// change the stat of the button's
	sendButton.setEnabled(true);
	cancelButton.setEnabled(false);
	saveButton.setEnabled(false);
	// Reload information
	getInformation();
    }
}
